@@ -23,10 +23,17 @@
  */
 #include <QtGui/QApplication>
 #include "tttfhgui.h"
+#include <QLocale>
+#include <QTranslator>
 
 int main(int argc, char *argv[])
 {
     QApplication a(argc, argv);
+
+    QTranslator myappTranslator;
+    myappTranslator.load(":langs/tttfh_" + QLocale::system().name());
+         a.installTranslator(&myappTranslator);
+
     TTTFHGui w;
     w.show();
     return a.exec();
