@@ -28,7 +28,7 @@ SoftGameArea::SoftGameArea(QWidget *parent, SessionDialog *sessionDialog, QLabel
     //QGLWidget(QGLFormat(QGL::SampleBuffers), parent)
         QWidget(parent)
 {
-    setAttribute(Qt::WA_OpaquePaintEvent,true);
+    //setAttribute(Qt::WA_OpaquePaintEvent,true);
     size=300;
     setFixedSize(size+size/6,size+size/6);
     setAutoFillBackground(false);
@@ -37,6 +37,9 @@ SoftGameArea::SoftGameArea(QWidget *parent, SessionDialog *sessionDialog, QLabel
 
     currentSet=0;
 
+    isOnline=0;
+    tnet=0;
+
     mode=0;
 
     board=0;
@@ -83,20 +86,76 @@ void SoftGameArea::animate(){
     //update();
 }
 
+void SoftGameArea::sendLogout(){
+    if(tnet)tnet->sendLogout();
+    endNetGame();
+}
+
 QString SoftGameArea::getStatus(){
     QString lMove("[[");
     QString rMove("]]");
-    QString wsize(QString::number(sessionDialog->getWinSize()));
+    QString wsize(QString::number(isOnline?winsize:sessionDialog->getWinSize()));
     QString cset(QString::number(currentSet+1));
-    QString tset(QString::number(sessionDialog->getSetsNum()));
-    return "WIN SIZE:"+wsize+" - SET:"+cset+"/"+tset+" - "+
-            (!board->who_plays()?lMove:"") + sessionDialog->getP1Name()+(sessionDialog->isHuman(0)?"":QString("(")+sessionDialog->getStringIA1()+")")+(!board->who_plays()?rMove:"")+" VS. "+
-            (board->who_plays()?lMove:"") + sessionDialog->getP2Name()+(sessionDialog->isHuman(1)?"":QString("(")+sessionDialog->getStringIA2()+")")+(board->who_plays()?rMove:"");
+    QString tset(QString::number(isOnline?sets:sessionDialog->getSetsNum()));
+    QString p1name(isOnline?this->p1name:sessionDialog->getP1Name());
+    QString p2name(isOnline?this->p2name:sessionDialog->getP2Name());
+    return tr("WIN SIZE:")+wsize+" - " + tr("SET:") +cset+"/"+tset+" - "+
+            (!board->who_plays()?lMove:"") + p1name +(sessionDialog->isHuman(0)?"":QString("(")+sessionDialog->getStringIA1()+")")+(!board->who_plays()?rMove:"")+" VS. "+
+            (board->who_plays()?lMove:"") + p2name +(sessionDialog->isHuman(1)?"":QString("(")+sessionDialog->getStringIA2()+")")+(board->who_plays()?rMove:"");
+}
+
+void SoftGameArea::newNetGame(int dim, int sets, bool newContext, TrisNet *tnet, QString p1name, QString p2name){
+    this->tnet=tnet; //Copying the network instance utility class
+
+    //If I am the master then I am also the p1(with the circle as sign)
+    isOnline=1+(!tnet->iAmMaster());
+
+    this->p1name=p1name;
+    this->p2name=p2name;
+
+    this->dim=dim;
+    this->winsize=dim==3||dim==5?3:dim==7?5:6;
+    this->sets=sets;
+
+    if(newContext && tnet->iAmMaster()){
+        //In this case we have to choose who is the first player of the first set
+        firstPlayer=qrand()>(RAND_MAX/2);
+        //Now we have to tell to the other player(slave) about the decision
+        tnet->sendFirstToMove(firstPlayer);
+    }
+
+    if(!newContext)synced=false;
+    else{
+        synced=true;
+        connect(tnet,SIGNAL(setSync()),SLOT(syncing()));
+        connect(tnet,SIGNAL(chatMess(QString)),this,SIGNAL(chatMess(QString)));
+
+        //The peer seems to be dead so I have to close this session emitting a signal
+        connect(tnet,SIGNAL(deadPeer()),this,SLOT(endNetGame()));
+    }
+
+    connect(tnet, SIGNAL(moveReceived(QPoint)), this, SLOT(peerMoveArrived(QPoint)));
+
+
+    if(tnet->iAmMaster() || !newContext){
+        mode=1;
+        newGame(dim,winsize,newContext);
+    }
+    else connect(tnet, SIGNAL(firstReceived(bool)), this, SLOT(startAsSlave(bool)));
+}
+
+void SoftGameArea::syncing(){
+    synced=true;
+}
+
+void SoftGameArea::startAsSlave(bool first){    
+    disconnect(tnet, SIGNAL(firstReceived(bool)), 0, 0);
+    this->firstPlayer=first;
+    mode=1;
+    newGame(dim,winsize,true);
 }
 
 void SoftGameArea::newGame(int dim, int winsize, bool newContext){
-
-
     if(newContext) currentSet=0;
 
     this->dim=dim;
@@ -193,7 +252,7 @@ void SoftGameArea::newGame(int dim, int winsize, bool newContext){
 
     winnerPainted=false;
 
-    if(newContext) firstPlayer=qrand()>(RAND_MAX/2);
+    if(newContext && !isOnline) firstPlayer=qrand()>(RAND_MAX/2);
     if((currentSet+firstPlayer)%2) board->cambia_turno();
 
     update();
@@ -207,14 +266,14 @@ void SoftGameArea::quickSettings(){
 void SoftGameArea::paintEvent(QPaintEvent *event){
     QPainter painter(this);
 
-    painter.fillRect(event->rect(), background);
+    //painter.fillRect(event->rect(), background);
 
-    QPen pen1(Qt::lightGray, 0.2, Qt::SolidLine);
-    painter.setPen(pen1);
-    for(int i=0;i<size+100;i+=11){
+//    QPen pen1(Qt::lightGray, 0.2, Qt::SolidLine);
+  //  painter.setPen(pen1);
+    /*for(int i=0;i<size+100;i+=11){
         painter.drawLine(5,i,size+size/6-5,i);
         painter.drawLine(i,5,i,size+size/6-5);
-    }
+    }*/
     painter.translate(25,25);
 
     painter.setRenderHint(QPainter::Antialiasing);
@@ -224,7 +283,7 @@ void SoftGameArea::paintEvent(QPaintEvent *event){
     painter.setPen(pen);
 
     if(!initialized){
-        srand((int)&painter);
+        //srand((int)&painter);
         for(int i=1;i<dim;++i){
             int aux=(size/dim)*i+qrand()/(float)RAND_MAX*(size/dim/3);
             int aux2=(size/dim)*i+qrand()/(float)RAND_MAX*(size/dim/3);
@@ -286,7 +345,7 @@ void SoftGameArea::paintEvent(QPaintEvent *event){
         painter.drawPixmap(0,0,QPixmap(":main/quick.png"));
         pen.setColor(Qt::magenta);
         painter.setPen(pen);
-        painter.drawText(10,size+size/15,"CLICK ON A RULE SET");
+        painter.drawText(10,size-size/dim/15,size+40,30,Qt::AlignHCenter||Qt::AlignCenter,tr("CLICK ON A RULE SET"));
         break;
     case 1:
         if(!animating)drawPoints(&painter);
@@ -302,26 +361,28 @@ void SoftGameArea::paintEvent(QPaintEvent *event){
             }
         }
 
-        if(!timer->isActive() && !animating && !board->who_plays() && !sessionDialog->isHuman(0)  && !symAnimating && !symAnimating2 && !board->finished()){
-            timer->stop();
-            int x,y;
-            x=TrisMachine::get_move(*board,y,false,ial1);
-            if(board->contrassegna(x,y,false)){
-                update();
-                return;
+        if(!timer->isActive() && !animating && !board->who_plays() && (!sessionDialog->isHuman(0) || isOnline)  && !symAnimating && !symAnimating2 && !board->finished()){
+            if(!isOnline){
+                timer->stop();
+                int x,y;
+                x=TrisMachine::get_move(*board,y,false,ial1);
+                if(board->contrassegna(x,y,false)){
+                    update();
+                    return;
+                }
             }
-                //drawO(getCenter(QPoint(y,x)),size/dim/4,&painter);
         }
 
-        if(!timer->isActive() && !animating && board->who_plays() && !sessionDialog->isHuman(1)  && !symAnimating && !symAnimating2 && !board->finished()){
-            timer->stop();
-            int x,y;
-            x=TrisMachine::get_move(*board,y,true,ial2);
-            if(board->contrassegna(x,y,true)){
-                update();
-                return;
+        if(!timer->isActive() && !animating && board->who_plays() && (!sessionDialog->isHuman(1) || isOnline) && !symAnimating && !symAnimating2 && !board->finished()){
+            if(!isOnline){
+                timer->stop();
+                int x,y;
+                x=TrisMachine::get_move(*board,y,true,ial2);
+                if(board->contrassegna(x,y,true)){
+                    update();
+                    return;
+                }
             }
-                //drawX(getCenter(QPoint(y,x)),size/dim/3,&painter);
         }
 
 
@@ -334,7 +395,7 @@ void SoftGameArea::paintEvent(QPaintEvent *event){
             QFont font;
             font.setPixelSize(15);
             painter.setFont(font);
-            painter.drawText(0,size+size/15,"CLICK TO CONTINUE");
+            painter.drawText(0,size+size/15,tr("CLICK TO CONTINUE"));
 
             if(!(~(*board) || !(*board))){
                 tie=true;
@@ -354,10 +415,14 @@ void SoftGameArea::paintEvent(QPaintEvent *event){
 
         break;
     }
+    bar->setText(getStatus());
+}
 
-
-    //(&painter);
-
+void SoftGameArea::peerMoveArrived(QPoint p){
+    if(board->contrassegna(p.x(),p.y(),isOnline==1)){
+        update();
+        return;
+    }
 }
 
 void SoftGameArea::mouseReleaseEvent(QMouseEvent *event){
@@ -392,34 +457,83 @@ void SoftGameArea::mouseReleaseEvent(QMouseEvent *event){
                 else if((~(*board))) points1++;
                 currentSet++;
                 int diff=(points1-points2)>=0?points1-points2:points2-points1;
-                int remaining=sessionDialog->getSetsNum()-currentSet;
-                if(currentSet>=sessionDialog->getSetsNum() || diff>remaining){
-                    QMessageBox::information(this, tr("Session finished"),
-                                             (points1>points2?sessionDialog->getP1Name():(points2>points1?sessionDialog->getP2Name():
-                                             tr("Nobody")))+tr(" wins this Game Session!"),
-                                             QMessageBox::Ok);
-                    currentSet=0;
-                    //newGame(sessionDialog->getGridSize(),sessionDialog->getWinSize(),true);
-                    quickSettings();
+                int remaining=isOnline?sets:sessionDialog->getSetsNum()-currentSet;
+                if(!isOnline){
+                    if(currentSet>=sessionDialog->getSetsNum() || diff>remaining){
+                        QMessageBox::information((QWidget*)this->parent()->parent(), tr("Session finished"),
+                                                 (points1>points2?sessionDialog->getP1Name():(points2>points1?sessionDialog->getP2Name():
+                                                                                              tr("Nobody")))+tr(" wins this Game Session!"),
+                                                 QMessageBox::Ok);
+                        currentSet=0;
+                        //newGame(sessionDialog->getGridSize(),sessionDialog->getWinSize(),true);
+                        quickSettings();
+                    }else{
+                        newGame(this->dim, this->winsize,false);
+                    }
                 }else{
-                    newGame(this->dim, this->winsize,false);
+                    //I have to send the results to the server on THIS channel
+                    //(Server side can check data because everyone sends its own copy of results)
+                    tnet->sendResults(points1,points2);
+
+                    if(currentSet>=sets || diff>remaining){
+                        //emit endGame(true);
+                        QMessageBox::information((QWidget*)this->parent()->parent(), tr("Match finished"),
+                                                 (points1>points2?p1name:points2>points1?p2name:tr("Nobody"))+tr(" wins this Game Session!"),
+                                                 QMessageBox::Ok);
+                        currentSet=0;
+
+                        //if(tnet->iAmMaster()){
+                            //In this case I have to tell to the server who wins
+                            //tnet->sendResults(points1,points2);
+                        //}
+
+                        //emit endGame();
+
+                        endNetGame();
+
+                    }else{
+                        synced=false;
+                        newGame(this->dim, this->winsize,false);
+                        tnet->sendSetSync();
+                    }
                 }
             }
             return;
         }
-        if(sessionDialog->isHuman(0) && !board->who_plays()){
-            board->contrassegna(slot.x(),slot.y(),false);
-        }else if(sessionDialog->isHuman(1) && board->who_plays()){
-            board->contrassegna(slot.x(),slot.y(),true);
+        if(((isOnline==1&&synced) || (isOnline==0&&sessionDialog->isHuman(0))) && !board->who_plays()){
+            bool moved=board->contrassegna(slot.x(),slot.y(),false);
+            if(isOnline && moved)
+                tnet->sendMove(QPoint(slot.x(),slot.y()));
+        }else if(((isOnline==2&&synced) || (isOnline==0&&sessionDialog->isHuman(1))) && board->who_plays()){
+            bool moved=board->contrassegna(slot.x(),slot.y(),true);
+            if(isOnline && moved)
+                tnet->sendMove(QPoint(slot.x(),slot.y()));
         }
 
         break;
     }
 
-
     update();
 }
 
+void SoftGameArea::endNetGame(){
+    emit endGame(true);
+    //Always when match finishes, the game returns to offline mode
+    //The general status obviously remains online(green led)
+    isOnline=0;
+
+    disconnect(tnet, SIGNAL(moveReceived(QPoint)), 0, 0);
+
+    //Now the TrisNet instance is not used anymore so I can delete it
+    delete tnet;
+    quickSettings();
+
+}
+
+void SoftGameArea::sendChatMessage(QString mess){
+    tnet->sendChatMess((tnet->iAmMaster()?p1name:p2name)+QString(";")+mess);
+}
+
 QPoint SoftGameArea::getCenter(QPoint slot){
     return QPoint(((size/dim))*slot.x()+(size/dim)/2,(((size/dim))*slot.y()+(size/dim)/2));
 }
@@ -618,7 +732,7 @@ void SoftGameArea::drawText(QPainter *painter, int index, QString text, QPointF
 
             QPolygonF poly(dText.toFillPolygon());
 
-            nTextPoints->push(poly.size());
+            nTextPoints->push(text==QString("0")?poly.size()/2-1:poly.size());
 
             QPointF *poly2=new QPointF[nTextPoints->top()];
             textPoints->push(poly2);
@@ -627,7 +741,6 @@ void SoftGameArea::drawText(QPainter *painter, int index, QString text, QPointF
             if(!timer->isActive())timer->start(25);
 
 
-
             int mem1,mem2,aux1=0,aux2=0,aux21=0,aux22=0;
             for (int i = 1; i < nTextPoints->top()-1; i++) {
                 mem1=qrand()/(float)RAND_MAX*(1.3+0.5*i/nTextPoints->top());
